#include "combinationiterator.hpp"

using namespace gdsl;

CombinationIterator::CombinationIterator(GDSLRestList &rests)
{
    for(auto rest : rests)
    {
        m_begins.push_back(rest->begin());
        m_its.push_back(rest->begin());
        m_ends.push_back(rest->end());
    }

    m_rests = rests;

    if(m_rests.size() == 0)
        m_end = true;

    for(auto rest : m_rests)
        if(rest->length() == 0)
            m_end = true;
}

CombinationIterator::CombinationIterator(std::vector<std::shared_ptr<GDSLRange> > &ranges)
{
    for(auto range : ranges)
    {
        m_begins.push_back(range->begin());
        m_its.push_back(range->begin());
        m_ends.push_back(range->end());
    }

    m_end = false;
}

GDSLRestList CombinationIterator::current()
{
    int i = 0;
    GDSLRestList singleRests;

    for(auto rest : m_rests)
    {
        singleRests.push_back(rest->singleRestAt(m_its.at(i)));
        ++i;
    }

    return singleRests;
}

std::vector<int> CombinationIterator::currentAsInt()
{
    std::vector<int> currentComb;

    for(auto its : m_its)
        currentComb.push_back(*its);

    return currentComb;
}

void CombinationIterator::reset()
{
    std::copy(m_begins.begin(), m_begins.end(), m_its.begin());
    m_end = false;
}

bool CombinationIterator::isEnd()
{
    return m_end;
}

void CombinationIterator::next()
{
    std::vector<GDSLRangeIterator>::iterator it_begin = m_begins.end() - 1;
    std::vector<GDSLRangeIterator>::iterator it_it = m_its.end() - 1;
    std::vector<GDSLRangeIterator>::iterator it_end = m_ends.end() - 1;

    for(; it_it >= m_its.begin(); --it_it, --it_end, --it_begin)
    {
        if((*it_it) != (*it_end) - 1)
        {
            (*it_it)++;
            break;
        }
        else
        {
            if(it_it == m_its.begin())
                m_end = true;

            (*it_it) = (*it_begin);
        }
    }
}
