#include "gdslassign.hpp"

#include "../parseexception.hpp"

using namespace gdsl;

void GDSLAssign::eval(GDSLVariable::GDSLVarTable &scope, ConstantsMap &constants, GDSLExpr::EquationList &constraints, std::shared_ptr<RandomAccessGraph> attr, std::shared_ptr<Grid> grid)
{
    auto left = m_left->eval(scope, constants, attr, grid);
    auto right = m_right->eval(scope, constants, attr, grid);

    auto eq = std::make_shared<IlpEquation>(left, right, m_op);
    constraints.push_back(eq);
}

bool GDSLAssign::parseFromTree(pANTLR3_BASE_TREE tree, GDSLVariable::GDSLVarTable& vars)
{
    int currentLine = tree->getLine(tree);
    bool leftSet = false;

    std::function<bool (pANTLR3_BASE_TREE)> parse = [&] (pANTLR3_BASE_TREE tree)
    {
        pANTLR3_COMMON_TOKEN tok = tree->getToken(tree);

        switch(tok->type)
        {
        case MATH:
        {
            if(!leftSet)
            {
                m_left = std::make_shared<MathExpr>(tree, vars);
                leftSet = true;
            }
            else
                m_right = std::make_shared<MathExpr>(tree, vars);
            return true;
        }
        default:
        {
            throw ParseException(antlrutils::getText(tree), currentLine);
        }
        }
    };

    antlrutils::foreachChildDo(tree, parse);

    return false;
}

std::string GDSLAssign::toString() const
{
    std::stringstream ss;

    //ss << m_left.toString() << " " << m_op.toString() << " " << m_right.toString();

    return ss.str();
}
