#ifndef gdslexpr_hpp
#define gdslexpr_hpp 1

#include <string>
#include <unordered_map>
#include <memory>
#include <functional>
#include <sstream>

#include "../GDSLGrammarLexer.h"
#include "../GDSLGrammarParser.h"
#include "../gdslvariable.hpp"
#include "../combinationiterator.hpp"
#include "../ilp/ilpexpr.hpp"
#include "../ilp/ilpequation.hpp"
#include "../grid.hpp"

#include "../randomaccessgraph.hpp"

#include <gurobi_c++.h>

namespace gdsl
{
class GDSLExpr 
{     
public:
    typedef std::vector< std::shared_ptr<IlpEquation> > EquationList;
    typedef std::unordered_map<std::string, int> ConstantsMap;

    virtual void eval(GDSLVariable::GDSLVarTable& scope, ConstantsMap& constants, EquationList& constraints, std::shared_ptr<RandomAccessGraph> attr,
                      std::shared_ptr<Grid> grid) = 0;
    virtual bool parseFromTree(pANTLR3_BASE_TREE tree, GDSLVariable::GDSLVarTable& vars) = 0;

    virtual std::string toString() const = 0;

    virtual void clear() = 0;

    std::string generateId(std::shared_ptr<GDSLVariable>& var)
    {
        std::string id = "";

        std::vector<GDSLRangeIterator> rests;
        for(auto rest : var.get()->restrictions())
            rests.push_back(rest->begin());

        id = generateId(rests);

        return id;
    }

    std::string generateId(std::vector<GDSLRangeIterator>& rests)
    {
        std::stringstream id;
        for(auto rest : rests)
            id << "_" << (*rest);

        return id.str();
    }

    //TODO: Remove rests from arguments
    void foreachRestDo(GDSLRestList& rests, std::string& iterationVarName, std::string& loopVarName,
                       GDSLVariable::GDSLVarTable& scope, ConstantsMap& constants, std::function<void (GDSLVariable::GDSLVarTable&, ConstantsMap& constants, std::shared_ptr<GDSLVariable>&)> func)
    {
        CombinationIterator combinations(rests);
        std::shared_ptr<GDSLVariable> scopeVar = nullptr;
        scopeVar = (scope.find(iterationVarName))->second;

        while(!combinations.isEnd())
        {
            GDSLRestList singleRests = combinations.current();
            std::shared_ptr<GDSLVariable> var = std::make_shared<GDSLVariable>(loopVarName, singleRests);

            GDSLVariable::IlpVarList vars = scopeVar->get(singleRests);
            var->setIlpVariables(vars);

            func(scope, constants, var);

            combinations.next();
        }

    }

private:

};
}
#endif // gdslexpr_hpp
