#include "gdslloop.hpp"

#include "gdslprogram.hpp"
#include "../parseexception.hpp"

using namespace gdsl;

void GDSLLoop::eval(GDSLVariable::GDSLVarTable &scope, ConstantsMap& constants, EquationList &constraints, std::shared_ptr<RandomAccessGraph> attr, std::shared_ptr<Grid> grid)
{
    //TODO: FIX THIS MESS!
    std::string origVarName;
    std::string iterName;
    GDSLRestList rests;

    if(m_varName != "")
    {
        if(scope.find(m_varName) == scope.end())
            throw ParseException("Unknown Variable " + m_varName, m_tree->getLine(m_tree));

        std::shared_ptr<GDSLVariable> orig = scope[m_varName];

        auto variable = std::make_shared<GDSLVariable>(m_tree, orig, scope);

        origVarName = variable->id();
        iterName = m_iterVar->id();
        rests = variable->restrictions();

        variable->apply(scope, constants, attr, grid);
    }

    else
    {
        if(scope.find(m_var->id()) == scope.end())
            throw ParseException("Variable unknown", m_tree->getLine(m_tree));

        origVarName = m_var->id();
        iterName = m_iterVar->id();
        rests = m_var->restrictions();

        m_var->apply(scope, constants, attr, grid);
    }


    std::function<void (GDSLVariable::GDSLVarTable&, ConstantsMap&, std::shared_ptr<GDSLVariable>&)> evalBody = [&] (GDSLVariable::GDSLVarTable& scope, ConstantsMap& constants, std::shared_ptr<GDSLVariable>& var)
    {
        scope[iterName] = var;
        m_body->eval(scope, constants, constraints, attr, grid);
        scope.erase(iterName);
    };

    foreachRestDo(rests, origVarName, iterName, scope, constants, evalBody);
}

bool GDSLLoop::parseFromTree(pANTLR3_BASE_TREE tree, GDSLVariable::GDSLVarTable& vars)
{
    int currentLine = tree->getLine(tree);
    bool iterSet = false;

    std::function<bool (pANTLR3_BASE_TREE)> parse = [&] (pANTLR3_BASE_TREE tree)
    {
        pANTLR3_COMMON_TOKEN tok = tree->getToken(tree);

        switch(tok->type)
        {
        case NAME:
        {
            if(iterSet)
            {
                m_varName = antlrutils::getText(tree);
                m_tree = tree;
            }
            else
            {
                m_iterVar = std::make_shared<GDSLVariable>(tree, nullptr, vars);

                if(vars.find(m_iterVar->id()) != vars.end())
                    throw ParseException(" variable " + m_iterVar->id() + " already defined, please choose another name!", currentLine);

                iterSet = true;
            }

            return true;
        }
        case FOO:
        {
            m_var = std::make_shared<GDSLVariable>(tree, nullptr, vars);
            return true;
        }
        case NODES:
        {
            if(!iterSet)
                return false;

            std::string name = GDSLProgram::GDSLNODES;
            m_var = std::make_shared<GDSLVariable>(tree, vars[name], vars);
            m_var->setName(name);

            return true;
        }
        case EDGES:
        {
            if(!iterSet)
                return false;

            std::string name = GDSLProgram::GDSLEDGES;
            m_var = std::make_shared<GDSLVariable>(tree, vars[name], vars);
            m_var->setName(name);

            return true;
        }
        case COLUMNS:
        {
            if(!iterSet)
                return false;

            std::string name = GDSLProgram::GDSLCOLUMNS;
            m_var = std::make_shared<GDSLVariable>(tree, vars[name], vars);
            m_var->setName(name);

            return true;
        }
        case ROWS:
        {
            if(!iterSet)
                return false;

            std::string name = GDSLProgram::GDSLROWS;
            m_var = std::make_shared<GDSLVariable>(tree, vars[name], vars);
            m_var->setName(name);

            return true;
        }
        case OBJECTS:
        {
            if(!iterSet)
                return false;

            std::string name = GDSLProgram::GDSLOBJECTS;
            m_var = std::make_shared<GDSLVariable>(tree, vars[name], vars);
            m_var->setName(name);

            return true;
        }
        case INTVL:
        {
            if(!iterSet)
                return false;

            std::string name = GDSLProgram::GDSLBIGGESTINTVL;
            m_var = std::make_shared<GDSLVariable>(tree, vars[name], vars);
            m_var->setName(name);

            return true;
        }
        case DO:
        {
            m_body = std::make_shared<GDSLProgram>(tree, vars);
            return true;
        }
//        case OBJREST:
//        {
//            if(!iterSet)
//                return false;

//            m_var = std::make_shared<GDSLVariable>(tree, nullptr, vars);
////            m_var->setName(name);

//            return true;
//        }
        default:
        {
            if(tok->type == IN || tok->type == END)
                return true;

            throw ParseException(antlrutils::getText(tree) + " while building loop", currentLine);
        }
        }
    };

    antlrutils::foreachChildDo(tree, parse);

    return true;
}

std::string GDSLLoop::toString() const
{
    std::stringstream ss;

    ss << "for " << m_iterVar.get()->toString() << " in "
       << (m_varName != "" ? m_varName : m_var->toString()) << " do" << std::endl;
    ss << m_body.get()->toString();
    ss << "end";

    return ss.str();
}

void GDSLLoop::clear()
{
     m_body->clear();

     if(m_var != nullptr)
        m_var->clearIlpVars();

     m_iterVar->clearIlpVars();
}
