#ifndef gdslprogram_hpp
#define gdslprogram_hpp 1

#include <vector>
#include <unordered_map>
#include <iostream>
#include <string>
#include <sstream>
#include <functional>
#include <memory>

#include "../OGDF/ogdf/basic/GraphAttributes.h"

#include "../antlrutilities.hpp"

#include "../gdslvariable.hpp"
#include "../operator.hpp"

#include "gdslloop.hpp"
#include "gdslassign.hpp"
#include "gdslexpr.hpp"

#include "../ilp/ilpequation.hpp"

#include <gurobi_c++.h>

#include "../GDSLGrammarLexer.h"
#include "../GDSLGrammarParser.h"

namespace gdsl
{
class GDSLProgram : public GDSLExpr
{
public:
    static const std::string GDSLNODES;
    static const std::string GDSLEDGES;
    static const std::string GDSLCOLUMNS;
    static const std::string GDSLROWS;
    static const std::string GDSLBIGGESTINTVL;
    static const std::string GDSLOBJECTS;
    static const std::vector<std::string> DEFAULT_VARS;

    static bool isDefaultVar(std::shared_ptr<GDSLVariable> var);

    GDSLProgram(std::string& name) {m_name = name;}
    GDSLProgram(std::string& name, std::string& program);

    GDSLProgram(pANTLR3_BASE_TREE tree) {m_name = ""; parseFromTree(tree, m_variables);}
    GDSLProgram(pANTLR3_BASE_TREE tree, GDSLVariable::GDSLVarTable& vars) {m_name = ""; parseFromTree(tree, vars);}
    GDSLProgram(pANTLR3_BASE_TREE tree, GDSLVariable::GDSLVarTable& vars, std::string name) {m_name = name; parseFromTree(tree, vars);}
    ~GDSLProgram();

    bool parseFromTree(pANTLR3_BASE_TREE tree) {return parseFromTree(tree, m_variables);}
    bool parseFromTree(pANTLR3_BASE_TREE tree, GDSLVariable::GDSLVarTable& vars);
    void eval(GDSLVariable::GDSLVarTable& scope, ConstantsMap& constants, EquationList& constraints, std::shared_ptr<RandomAccessGraph> attr,
              std::shared_ptr<Grid> grid);
    void eval();
    void runForGraph(std::shared_ptr<RandomAccessGraph>  attr, std::shared_ptr<Grid> grid);

    std::string name() const {return m_name;}
    std::string toString() const;

    std::shared_ptr<GRBModel> toGurobi(GRBEnv& env) const;
    std::string grbToString() const;
    std::vector<std::shared_ptr<GDSLVariable>> const variables();
    void clear();

    void setConstants(ConstantsMap& constants) {m_constants = constants;}

private:
    template<class OperatorType>
    bool createAssign(pANTLR3_BASE_TREE tree, GDSLVariable::GDSLVarTable& vars);

    std::shared_ptr<RandomAccessGraph> m_attr;
    std::shared_ptr<Grid> m_grid;
    std::vector< std::shared_ptr<GDSLExpr> > m_program;
    GDSLVariable::GDSLVarTable m_variables;
    ConstantsMap m_constants;
    std::string m_name;
    std::string m_programStr;

    GDSLGrammarParser_ilp_return m_r;
    pANTLR3_BASE_TREE  m_tree = nullptr;
    pANTLR3_INPUT_STREAM input = nullptr;
    pGDSLGrammarLexer lex = nullptr;
    pANTLR3_COMMON_TOKEN_STREAM tokens = nullptr;
    pGDSLGrammarParser parser = nullptr;
    pANTLR3_UINT8 inputStr = nullptr;
    pANTLR3_UINT8 nameStr = nullptr;

    EquationList m_constraints;
};
}
#endif // gdslprogram_hpp
