#ifndef MATHEXPR_HPP
#define MATHEXPR_HPP

#include <iostream>
#include <functional>
#include <sstream>
#include <string>
#include <vector>
#include <algorithm>
#include <memory>

#include "../GDSLGrammarLexer.h"
#include "../GDSLGrammarParser.h"

#include "../antlrutilities.hpp"
#include "../gdslvariable.hpp"
#include "sumexpr.hpp"

#include "gdsleqexpr.hpp"

#include "../ilp/ilpexpr.hpp"
#include "../ilp/ilpexprlist.hpp"

namespace gdsl
{
class MathExpr : public GDSLEqExpr
{   
public:
    MathExpr() {}
    MathExpr(std::shared_ptr<GDSLVariable> var);
    MathExpr(pANTLR3_BASE_TREE tree, GDSLVariable::GDSLVarTable& vars) {parseFromTree(tree, vars);}

    bool parseFromTree(pANTLR3_BASE_TREE tree, GDSLVariable::GDSLVarTable&);
    void eval(GDSLVariable::GDSLVarTable& scope, ConstantsMap& constants, EquationList&, std::shared_ptr<RandomAccessGraph> attr,
              std::shared_ptr<Grid> grid);

    std::string toString() const {return "";}

    std::shared_ptr<IlpExprList> eval(GDSLVariable::GDSLVarTable &scope, ConstantsMap& constants, std::shared_ptr<RandomAccessGraph> attr, std::shared_ptr<Grid> grid);
    void apply(std::shared_ptr<RandomAccessGraph>& attr, std::shared_ptr<Grid>& grid)
    {
        m_attr = attr;
        m_grid = grid;
    }

    void clear() {m_evaled = false;}

private:
    bool m_evaled;
    pANTLR3_BASE_TREE m_tree;
    std::shared_ptr<RandomAccessGraph> m_attr;
    std::shared_ptr<Grid> m_grid;
};

std::shared_ptr<IlpExprList> operator+(const std::shared_ptr<IlpExprList>& left, const std::shared_ptr<IlpExprList>& right);
std::shared_ptr<IlpExprList> operator%(const std::shared_ptr<IlpExprList>& left, const std::shared_ptr<IlpExprList>& right);
std::shared_ptr<IlpExprList> operator-(const std::shared_ptr<IlpExprList>& left, const std::shared_ptr<IlpExprList>& right);
std::shared_ptr<IlpExprList> operator*(const std::shared_ptr<IlpExprList>& left, const std::shared_ptr<IlpExprList>& right);
std::shared_ptr<IlpExprList> operator*(double mult, const std::shared_ptr<IlpExprList>& right);
std::shared_ptr<IlpExprList> operator*(const std::shared_ptr<IlpExprList>& right, double mult);
std::shared_ptr<IlpExprList> operator/(const std::shared_ptr<IlpExprList>& left, const std::shared_ptr<IlpExprList>& right);
bool operator<(const std::shared_ptr<IlpExprList>& left, const std::shared_ptr<IlpExprList>& right);
bool operator>(const std::shared_ptr<IlpExprList>& left, const std::shared_ptr<IlpExprList>& right);
}
#endif // MATHEXPR_HPP
