#include "sumexpr.hpp"

#include "../parseexception.hpp"

using namespace gdsl;

bool SumExpr::parseFromTree(pANTLR3_BASE_TREE tree, GDSLVariable::GDSLVarTable&)
{
    m_tree = tree;
    m_evaled = false;

    return true;
}

void SumExpr::eval(GDSLVariable::GDSLVarTable &scope, ConstantsMap &constants, GDSLExpr::EquationList&, std::shared_ptr<RandomAccessGraph> attr,
                   std::shared_ptr<Grid> grid)
{
    auto exprs = eval(scope, constants, attr, grid);
    return;
}

std::shared_ptr<IlpExprList> SumExpr::eval(GDSLVariable::GDSLVarTable &scope, ConstantsMap &constants, std::shared_ptr<RandomAccessGraph> attr, std::shared_ptr<Grid> grid)
{
    std::shared_ptr<GDSLVariable> var;

    //TODO: Should be parsed before eval
    //TODO: Exception handling
    std::function<bool(pANTLR3_BASE_TREE)> parse = [&] (pANTLR3_BASE_TREE tree)
    {
        pANTLR3_COMMON_TOKEN tok = tree->getToken(tree);
        if(tok)
        {
            switch(tok->type)
            {
                case NAME:
                {

                    std::string name = antlrutils::getText(tree);

                    if(scope.find(name) == scope.end())
                        throw ParseException("Unknown Variable " + name, tree->getLine(tree));

                    std::shared_ptr<GDSLVariable> orig = scope[name];
                    var = std::make_shared<GDSLVariable>(tree, orig, scope);

                    return true;
                }
                case SUM:
                {
                    if(parse(antlrutils::getChild(tree, 0)))
                        return true;

                    return false;
                }
                default:
                {

                    throw ParseException(antlrutils::getText(tree) + " while building sum expression.", tree->getLine(tree));
                }
            }
        }
    };

    antlrutils::foreachChildDo(m_tree, parse);
    var->apply(scope, constants, attr, grid);

    if(scope.find(var->id()) == scope.end())
        throw ParseException("Variable uknown", m_tree->getLine(m_tree));

    std::shared_ptr<GDSLVariable> scopeVar = scope.at(var->id());
    auto restrictions = var->restrictions();
    CombinationIterator combinations(restrictions);
    auto exprs = std::make_shared<IlpExprList>();
    auto sumExpr = std::make_shared<IlpExpr>(0);

    while(!combinations.isEnd())
    {
        GDSLRestList singleRests = combinations.current();

        auto ilpVars = scopeVar->get(singleRests);

        for(auto ilpVar : ilpVars)
        {
            auto expr = std::make_shared<IlpExpr>(0);
            expr->add(1, ilpVar);
            sumExpr = sumExpr + expr;
        }

        combinations.next();
    }

    exprs->append(sumExpr);
    return exprs;
}
