#ifndef gdslvariable_hpp
#define gdslvariable_hpp 1

#include <string>
#include <sstream>
#include <memory>
#include <unordered_map>

#include "GDSLGrammarParser.h"
#include "GDSLGrammarLexer.h"

#include "antlrutilities.hpp"
#include "restrictions/gdslattributerest.hpp"
#include "restrictions/gdslpropertyrestriction.hpp"
#include "restrictions/gdslintrest.hpp"
#include "ilp/ilpvariable.hpp"
#include "combinationiterator.hpp"
#include "gurobi_c++.h"
#include "randomaccessgraph.hpp"

namespace gdsl
{
class GDSLVariable : public std::enable_shared_from_this<GDSLVariable>
{     

public:
    typedef std::vector< std::shared_ptr<IlpVariable> > IlpVarList;
    typedef std::vector< std::shared_ptr<GDSLVariable> > GDSLVarVector;
    typedef std::unordered_map<std::string, std::shared_ptr<GDSLVariable> > GDSLVarTable;

    GDSLVariable() {}
    GDSLVariable(const std::string& name, GDSLRestList& rests);
    GDSLVariable(pANTLR3_BASE_TREE tree, std::shared_ptr<GDSLVariable> orig, GDSLVarTable &scope);

    bool parseFromTree(pANTLR3_BASE_TREE tree, std::shared_ptr<GDSLVariable> orig, GDSLVarTable& scope);
    std::string toString() const;
    std::string toString(int id) const;
    std::string id() const {return m_name;}
    void setName(std::string& name) {m_name = name;}

    GDSLRestList restrictions() const {return m_rest;}
    std::shared_ptr<GDSLRestriction> getRestrictionAt(int i) const {return m_rest.at(i);}
    bool restListIsSubset(const GDSLRestList& rest) const;

    IlpVarList get(GDSLRestList& rests);
    void createIlpVariables();
    void setIlpVariables(IlpVarList& vars) {m_ilpVars = vars;}
    IlpVarList ilpVars() const {return m_ilpVars;}
    void clearIlpVars();

    void createGRBVars(std::shared_ptr<GRBModel>& model);
    std::string grbToString() const;

    void apply(GDSLVarTable& scope, ConstantsMap& constants, std::shared_ptr<RandomAccessGraph>& attr, std::shared_ptr<Grid>& grid);

    std::shared_ptr<RandomAccessGraph> attr() {return m_attr;}
    std::shared_ptr<Grid> grid() {return m_grid;}

    int idFromRest(GDSLRestList& rests) const;
    std::shared_ptr<GDSLRestList> restFromId(int id) const;

private:
    std::string m_name;
    GDSLRestList m_rest;
    IlpVarList m_ilpVars;

    std::shared_ptr<RandomAccessGraph> m_attr;
    std::shared_ptr<Grid> m_grid;
};
}
#endif // gdslvariable_hpp
