#include "ilpequation.hpp"

using namespace gdsl;

std::string IlpEquation::toString() const
{
    std::stringstream ss;

    auto leftIt = m_left->begin();
    auto rightIt = m_right->begin();

    if(m_left->isNumber())
        for(; rightIt < m_right->end(); ++rightIt)
            ss << (*leftIt)->toString() << " " << m_op->toString() << " " << (*rightIt)->toString() << std::endl;

    else if(m_right->isNumber())
        for(; leftIt < m_left->end(); ++leftIt)
            ss << (*leftIt)->toString() << " " << m_op->toString() << " " << (*rightIt)->toString() << std::endl;

    else
        for(; leftIt < m_left->end() && rightIt < m_right->end(); ++leftIt, ++rightIt)
            ss << (*leftIt)->toString() << " " << m_op->toString() << " " << (*rightIt)->toString() << std::endl;

    return ss.str();
}

void IlpEquation::toGurobi(std::shared_ptr<GRBModel> &model) const
{
    auto leftIt = m_left->begin();
    auto rightIt = m_right->begin();

    if(m_left->isNumber())
        for(; rightIt < m_right->end(); ++rightIt)
            model->addConstr((*leftIt)->toGurobi(), m_op->toGurobi(), (*rightIt)->toGurobi());

    else if(m_right->isNumber())
        for(; leftIt < m_left->end(); ++leftIt)
        {
            auto left = (*leftIt)->toGurobi();
            auto right = (*rightIt)->toGurobi();
            auto op = m_op->toGurobi();
            model->addConstr(left, op, right);
        }

    else
        for(; leftIt < m_left->end() && rightIt < m_right->end(); ++leftIt, ++rightIt)
            model->addConstr((*leftIt)->toGurobi(), m_op->toGurobi(), (*rightIt)->toGurobi());
}
