#ifndef ILPEXPR_HPP
#define ILPEXPR_HPP

#include <iostream>
#include <vector>
#include <utility>
#include <string>
#include <sstream>

#include <gurobi_c++.h>

#include "ilpvariable.hpp"

namespace gdsl
{
class IlpExpr
{
    typedef std::pair< double, std::shared_ptr<IlpVariable> > IlpVarCoeffPair;
    typedef std::vector< IlpVarCoeffPair > IlpCoeffVarVector;
    typedef IlpCoeffVarVector::iterator IlpExprIterator;
    typedef IlpCoeffVarVector::const_iterator CIlpExprIterator;

public:
    IlpExpr() {}
    IlpExpr(double constant) {m_constant = constant;}
    IlpExpr(std::vector<double> coeffs, std::vector< std::shared_ptr<IlpVariable> > variables, int constant = 0);

    void add(IlpVarCoeffPair& pair) {m_vars.push_back(pair);}
    void add(double coeff, std::shared_ptr<IlpVariable>& var);
    void setConstant(double constant) {m_constant = constant;}
    double getConstant() const {return m_constant;}

    bool isNumber() const {return m_vars.empty();}
    int numVars() const {return m_vars.size();}

    IlpExprIterator begin() {return m_vars.begin();}
    IlpExprIterator end() {return m_vars.end();}

    CIlpExprIterator begin() const {return m_vars.begin();}
    CIlpExprIterator end() const {return m_vars.end();}

    std::string toString() const;
    GRBLinExpr toGurobi() const;

private:
    IlpCoeffVarVector m_vars;
    double m_constant;
};

std::shared_ptr<IlpExpr> operator+(const std::shared_ptr<IlpExpr>& left, const std::shared_ptr<IlpExpr>& right);
std::shared_ptr<IlpExpr> operator-(const std::shared_ptr<IlpExpr>& left, const std::shared_ptr<IlpExpr>& right);

std::shared_ptr<IlpExpr> operator*(const std::shared_ptr<IlpExpr>& left, const double right);
std::shared_ptr<IlpExpr> operator*(const double left, const std::shared_ptr<IlpExpr>& right);

std::shared_ptr<IlpExpr> operator/(const std::shared_ptr<IlpExpr>& left, const std::shared_ptr<IlpExpr>& right);

bool operator>(const std::shared_ptr<IlpExpr>& left, const std::shared_ptr<IlpExpr>& right);
bool operator<(const std::shared_ptr<IlpExpr>& left, const std::shared_ptr<IlpExpr>& right);
}
#endif // ILPEXPR_HPP
