#include "randomaccessgraph.hpp"
#include "math.h"

using namespace gdsl;

RandomAccessGraph::RandomAccessGraph(std::shared_ptr<ogdf::GraphAttributes> &attr)
{
    m_attr = attr;

    ogdf::node node;
    ogdf::edge edge;

    m_edgeIdMult = pow10(ceil(log10(attr->constGraph().numberOfNodes())));
    std::cout << m_edgeIdMult << std::endl;

    forall_nodes(node, attr->constGraph())
    {
        m_nodes.push_back(node);
        m_nodeToId[node] = m_nodes.size() - 1;
    }

    forall_edges(edge, attr->constGraph())
    {
        m_edges.push_back(edge);
        m_edgeToId[edge] = m_edges.size() - 1;
    }
}

ogdf::node RandomAccessGraph::nodeById(int id)
{
    int realId = id - 1;

    if(realId < m_nodes.size())
        return m_nodes[realId];

    return m_nodes[0];
}

ogdf::edge RandomAccessGraph::edgeById(int id)
{
    int realId = id - m_edgeIdMult - 1;

    if(realId < m_edges.size())
        return m_edges[realId];

    return m_edges[0];
}

int RandomAccessGraph::idByNode(ogdf::node node)
{
    if(m_nodeToId.find(node) != m_nodeToId.end())
            return m_nodeToId[node] + 1;

    return 0;
}

int RandomAccessGraph::idByEdge(ogdf::edge edge)
{
    if(m_edgeToId.find(edge) != m_edgeToId.end())
            return m_edgeIdMult + m_edgeToId[edge] + 1;

    return 0;
}

