#ifndef GDSLATTRIBUTEREST_HPP
#define GDSLATTRIBUTEREST_HPP

#include "gdslrestriction.hpp"

namespace gdsl
{

class GDSLAttributeRest;

typedef std::function<void (ogdf::node)> NodeFunction;
typedef std::function<void (ogdf::edge)> EdgeFunction;

std::shared_ptr<GDSLAttributeRest> makeNodeNullRestriction(pANTLR3_BASE_TREE tree, GDSLVarTable& scope);
std::shared_ptr<GDSLAttributeRest> makeEdgeNullRestriction(pANTLR3_BASE_TREE tree, GDSLVarTable& scope);
std::shared_ptr<GDSLAttributeRest> makeObjectNullRestriction(pANTLR3_BASE_TREE tree, GDSLVarTable& scope);
std::shared_ptr<GDSLAttributeRest> makeNameNullRestriction(pANTLR3_BASE_TREE tree, GDSLVarTable& scope);

std::shared_ptr<GDSLAttributeRest> makeNodeColorRestriction(pANTLR3_BASE_TREE tree, GDSLVarTable& scope);
std::shared_ptr<GDSLAttributeRest> makeEdgeColorRestriction(pANTLR3_BASE_TREE tree, GDSLVarTable& scope);

std::shared_ptr<GDSLAttributeRest> makeNegatedNodeColorRestriction(pANTLR3_BASE_TREE tree, GDSLVarTable& scope);
std::shared_ptr<GDSLAttributeRest> makeNegatedEdgeColorRestriction(pANTLR3_BASE_TREE tree, GDSLVarTable& scope);

std::shared_ptr<GDSLAttributeRest> makeNodeLabelRestriction(pANTLR3_BASE_TREE tree, GDSLVarTable& scope);
std::shared_ptr<GDSLAttributeRest> makeEdgeLabelRestriction(pANTLR3_BASE_TREE tree, GDSLVarTable& scope);

std::shared_ptr<GDSLAttributeRest> makeNegatedNodeLabelRestriction(pANTLR3_BASE_TREE tree, GDSLVarTable& scope);
std::shared_ptr<GDSLAttributeRest> makeNegatedEdgeLabelRestriction(pANTLR3_BASE_TREE tree, GDSLVarTable& scope);

std::shared_ptr<GDSLAttributeRest> makeWithoutNodesRestriction(pANTLR3_BASE_TREE tree, GDSLVarTable& scope);
std::shared_ptr<GDSLAttributeRest> makeWithoutEdgesRestriction(pANTLR3_BASE_TREE tree, GDSLVarTable& scope);

class GDSLAttributeRest : public GDSLRestriction
{
public:
    GDSLAttributeRest(pANTLR3_BASE_TREE tree, GDSLVarTable& scope, RestrictionType inputType, RestrictionType outputType) : GDSLRestriction(inputType, outputType) {m_range = std::make_shared<GDSLClosedIntvl>(); parseFromTree(tree, scope);}
    GDSLAttributeRest(GDSLRangeIterator& it, RestrictionType inputType, RestrictionType outputType) : GDSLRestriction(it, inputType, outputType) {}

    bool parseFromTree(pANTLR3_BASE_TREE tree, std::unordered_map< std::string, std::shared_ptr<GDSLVariable> >&);
    bool isSubset(const std::shared_ptr<GDSLRestriction>&) const {return true;}

    virtual std::shared_ptr<GDSLRestriction> singleRestAt(GDSLRangeIterator& it) const
    {
        auto apply = [] (std::weak_ptr<GDSLRestriction>, GDSLVarTable&, ConstantsMap&,
                std::shared_ptr<RandomAccessGraph>&, std::shared_ptr<Grid>&)
        {
            return true;
        };

        if(outputType() == RestrictionType::OBJECTREST)
        {
            if(it - begin() >= m_numNodes)
                return makeRestriction<GDSLRestriction>(apply, RestrictionType::EDGEREST, RestrictionType::EDGEREST, it);
            else
                return makeRestriction<GDSLRestriction>(apply, RestrictionType::NODEREST, RestrictionType::NODEREST, it);
        }

        return makeRestriction<GDSLRestriction>(apply, inputType(), outputType(), it);
    }

    virtual int pos(std::shared_ptr<GDSLRestriction> rest) const
    {
        if(outputType() == RestrictionType::OBJECTREST)
            if(rest->outputType() == RestrictionType::EDGEREST)
                return m_range->pos(*rest->begin(), (m_range->begin() + m_numNodes), m_range->end());

        return m_range->pos(*rest->begin(), m_range->begin(), m_range->end());
    }

    std::string toString() const {return m_range->toString();}

    std::string attribute() const {return m_attribute;}

    void setNumNodes(int n) {m_numNodes = n;}

    string colorName(string hexColor)
    {
        {
            hexColor = hexColor.erase(0, 1);

            int r = std::stoi(hexColor.substr(0,2), nullptr, 16);
            int g = std::stoi(hexColor.substr(2,2), nullptr, 16);
            int b = std::stoi(hexColor.substr(4,2), nullptr, 16);

            if(r < 64 && g < 64 && b < 64)
                return "black";

            if(r == 255 && g == 255 && b == 255)
                return "white";

            if(r >= 64 && g <= 51 && b <= 51)
                return "red";

            if(r <= 51 && g >= 64 && b <= 51)
                return "green";

            if(r <= 51 && g <= 51 && b >= 64)
                return "blue";

            if(r >= 64 && g >= 64 && b <= 51)
                return "yellow";

            if(r <= 51 && g >= 64 && b >= 64)
                return "teal";

            if(r >= 64 && g <= 51 && b >= 64)
                return "purple";

            return "gray";
        }
    }

protected:
    std::string m_attribute;
    int m_numNodes = 0;
};
}

#endif // GDSLATTRIBUTEREST_HPP
