#ifndef gdslintrest_hpp
#define gdslintrest_hpp 1

#include <iostream>
#include <memory>

#include "gdslrange.hpp"
#include "gdslclosedintvl.hpp"
#include "gdslrestriction.hpp"
#include "../antlrutilities.hpp"

namespace gdsl
{
class MathExpr;

class GDSLIntRest : public GDSLRestriction
{     

public:
    GDSLIntRest(pANTLR3_BASE_TREE tree, GDSLVarTable& scope, RestrictionType inputType, RestrictionType outputType) : GDSLRestriction(inputType, outputType) {parseFromTree(tree, scope);}

    bool parseFromTree(pANTLR3_BASE_TREE tree, std::unordered_map< std::string, std::shared_ptr<GDSLVariable> >& scope);
    bool isSubset(const std::shared_ptr<GDSLRestriction>&) const;

    std::string toString() const;

    int intbegin() const;
    int intend() const;

    void evalMathExpr(GDSLVarTable &scope, ConstantsMap& constants, std::shared_ptr<RandomAccessGraph> &attr, std::shared_ptr<Grid> &grid);

    std::vector<int>& mathPos() {return m_mathPos;}
    std::vector<int>& maxRowPos() {return m_maxRowPos;}
    std::vector<int>& maxColPos() {return m_maxColPos;}
    std::vector<std::shared_ptr<MathExpr>>& mathExpr() {return m_mathExpr;}
    bool contignous() const {return m_contignous;}

    void setAt(int pos, int value) {m_range->setAt(pos, value);}
    void setBeginEnd(int begin, int end) {m_range->setBeginEnd(begin, end);}
    void makeRangeContignous() {m_range->makeContignous();}

protected:
    std::vector<int> m_mathPos;
    std::vector< std::shared_ptr<MathExpr> > m_mathExpr;
    std::vector<int> m_maxRowPos;
    std::vector<int> m_maxColPos;
    bool m_contignous = false;
};

std::shared_ptr<GDSLIntRest> makeIntRestriction(pANTLR3_BASE_TREE tree, GDSLVarTable& scope);
std::shared_ptr<GDSLIntRest> makeMaxIntRestriction(pANTLR3_BASE_TREE tree, GDSLVarTable& scope);
std::shared_ptr<GDSLIntRest> makeRowRestriction(pANTLR3_BASE_TREE tree, GDSLVarTable& scope);
std::shared_ptr<GDSLIntRest> makeMaxRowRestriction(pANTLR3_BASE_TREE tree, GDSLVarTable& scope);
std::shared_ptr<GDSLIntRest> makeColumnRestriction(pANTLR3_BASE_TREE tree, GDSLVarTable& scope);
std::shared_ptr<GDSLIntRest> makeMaxColumnRestriction(pANTLR3_BASE_TREE tree, GDSLVarTable& scope);

}
#endif // gdslintrest_hpp

