#include "gdslpropertyrestriction.hpp"

#include "../gdslvariable.hpp"

using namespace gdsl;

std::shared_ptr<GDSLPropertyRestriction> gdsl::makeNeighbourRestriction()
{
    auto apply = [] (std::weak_ptr<GDSLPropertyRestriction> restwp, GDSLVarTable&, ConstantsMap&,
            std::shared_ptr<RandomAccessGraph> &attr, std::shared_ptr<Grid>&)
    {
        auto rest = restwp.lock();
        rest->clear();

        for(GDSLRangeIterator it = rest->innerRestriction()->begin(); it < rest->innerRestriction()->end(); ++it)
        {
            ogdf::node node = attr->nodeById(*it);
            ogdf::edge e;
            forall_adj_edges(e, node)
            {
                int id = attr->idByNode(e->opposite(node));
                rest->append(id);
            }
        }

        return true;
    };

    return makeRestriction<GDSLPropertyRestriction>(apply, RestrictionType::NODEREST, RestrictionType::NODEREST);
}


std::shared_ptr<GDSLPropertyRestriction> gdsl::makeSourceRestriction()
{
    auto apply = [] (std::weak_ptr<GDSLPropertyRestriction> restwp, GDSLVarTable&, ConstantsMap&,
            std::shared_ptr<RandomAccessGraph> &attr, std::shared_ptr<Grid>&)
    {
        auto rest = restwp.lock();
        rest->clear();

        for(GDSLRangeIterator it = rest->innerRestriction()->begin(); it < rest->innerRestriction()->end(); ++it)
        {
            ogdf::edge edge = attr->edgeById(*it);
            rest->append(attr->idByNode(edge->source()));
        }

        return true;
    };

    return makeRestriction<GDSLPropertyRestriction>(apply, RestrictionType::EDGEREST, RestrictionType::NODEREST);
}


std::shared_ptr<GDSLPropertyRestriction> gdsl::makeTargetRestriction()
{
    auto apply = [] (std::weak_ptr<GDSLPropertyRestriction> restwp, GDSLVarTable&, ConstantsMap&,
            std::shared_ptr<RandomAccessGraph>&attr, std::shared_ptr<Grid>&)
    {
        auto rest = restwp.lock();
        rest->clear();

        for(GDSLRangeIterator it = rest->innerRestriction()->begin(); it < rest->innerRestriction()->end(); ++it)
        {
            ogdf::edge edge = attr->edgeById(*it);
            rest->append(attr->idByNode(edge->target()));
        }

        return true;
    };

    return makeRestriction<GDSLPropertyRestriction>(apply, RestrictionType::EDGEREST, RestrictionType::NODEREST);
}


std::shared_ptr<GDSLPropertyRestriction> gdsl::makeIncidentEdgesRestriction()
{
    auto apply = [] (std::weak_ptr<GDSLPropertyRestriction> restwp, GDSLVarTable&, ConstantsMap&,
            std::shared_ptr<RandomAccessGraph> &attr, std::shared_ptr<Grid>&)
    {
        auto rest = restwp.lock();
        rest->clear();

        for(GDSLRangeIterator it = rest->innerRestriction()->begin(); it < rest->innerRestriction()->end(); ++it)
        {
            ogdf::node node = attr->nodeById(*it);
            ogdf::edge e;
            forall_adj_edges(e, node)
                rest->append(attr->idByEdge(e));
        }

        return true;
    };

    return makeRestriction<GDSLPropertyRestriction>(apply, RestrictionType::NODEREST, RestrictionType::EDGEREST);
}
