#ifndef EXECUTEFORSTMT_HPP
#define EXECUTEFORSTMT_HPP

#include "valuestmt.hpp"

namespace gdsl
{
class ExecuteForStmt : public ValueStmt
{
public:
    static std::shared_ptr<ExecuteForStmt> parseLine(std::string& ppLine, std::string& makroDir);

    ExecuteForStmt(std::string name, std::string start, std::string end) :
        ValueStmt(name)
    {
        m_start = PPUtils::toInt(start);
        m_stop = PPUtils::toInt(end);

        if(m_start > m_stop)
            throw PreprocessorException("start " + start + " bigger than end " + end + " in execute for");

        m_current = m_start;
    }

    virtual bool next()
    {
        if(m_current >= m_stop)
            return false;

        ++m_current;

        return true;
    }

    virtual int value() const {return m_current;}
    virtual int start() const {return m_start;}
    virtual int end() const {return m_stop;}
    virtual RET_TYPE returnType() const {return RET_TYPE::PPINT;}

private:
    std::string m_name;
    int m_start;
    int m_stop;
    int m_current;
};
}

#endif // EXECUTEFORSTMT_HPP
