#include "preprocessorutilities.hpp"

#include "executeforstmt.hpp"
#include "usestmt.hpp"

using namespace gdsl;

void PPUtils::collapseWhitespaces(std::string &ppLine)
{
    std::replace(ppLine.begin(), ppLine.end(), '\t', ' ');
    std::replace(ppLine.begin(), ppLine.end(), '\r', ' ');
    ppLine.erase(std::unique(ppLine.begin(), ppLine.end(), [] (char l, char r) {return (l == ' ' && r == ' ');}), ppLine.end());
}

void PPUtils::stripWhitespaces(std::string &str)
{
    while(*str.begin() == ' ')
        str.erase(0, 1);

    while(*(str.end() - 1) == ' ')
        str.erase(str.end() - 1);
}

std::string PPUtils::pop(std::string &ppLine, char delimiter, bool deleteDelimiter)
{
    std::string popped;

    auto it = ppLine.begin();
    for(; it < ppLine.end() && *it != delimiter; ++it)
        popped += *it;

    if(!deleteDelimiter)
        ppLine.erase(ppLine.begin(), it);
    else if(it + 1 < ppLine.end())
        ppLine.erase(ppLine.begin(), it + 1);
    else
        ppLine.erase(ppLine.begin(), ppLine.end());

    return popped;
}

char PPUtils::popFirst(std::string &ppLine)
{
    char popped = ppLine[0];
    ppLine.erase(0 , 1);

    return popped;
}

bool PPUtils::isName(const std::string &name)
{
    if(!std::isalpha(name.front()))
        return false;

    std::string sub = name.substr(1);
    if(!std::all_of(sub.begin(), sub.end(), [](char c) {return (std::isalnum(c) || c == '_');}))
        return false;

    return true;
}

bool PPUtils::isNumber(const std::string &number)
{
    if(std::all_of(number.begin(), number.end(), [](char c){return std::isdigit(c);}))
        return true;

    return false;
}

int PPUtils::toInt(const std::string &number)
{
    try
    {
        int integer = std::stoi(number);
        return integer;
    }
    catch(std::invalid_argument e)
    {
        throw PreprocessorException(number + " is no valid integer");
    }
}

bool PPUtils::isBool(const std::string &boolean)
{
    if(boolean == "true" || boolean == "false" || boolean == "0" || boolean == "1")
        return true;

    return false;
}

bool PPUtils::toBool(const std::string &boolean)
{
    std::cout << boolean << std::endl;
    if(boolean == "true" || boolean == "1")
        return true;
    else if(boolean == "false" || boolean == "0")
        return false;
    else
        throw PreprocessorException(boolean + " is no boolean");
}

void PPUtils::replaceAll(std::string &oldStr, std::string &newStr, std::string &program)
{
    size_t pos = program.find(oldStr);

    while(pos != program.npos)
    {
        bool beforeOk = false;
        bool afterOk = false;

        auto before = program.cbegin() + pos - 1;
        auto after = program.cbegin() + pos + oldStr.length();

        if(pos == 0)
            beforeOk = true;
        else if(!(std::isalnum(*before) || *before == '_'))
            beforeOk = true;

        if(pos == program.length() - 1)
            afterOk = true;
        else if(!(std::isalnum(*after) || *after == '_'))
            afterOk = true;

        if(beforeOk && afterOk)
        {
            program.erase(pos, oldStr.length());
            program.insert(pos, newStr);
        }

        ++pos;
        pos = program.find(oldStr, pos);
    }
}

std::string PPUtils::getName(std::string &ppLine)
{
    std::string name = pop(ppLine, '\n');

    return name;
}

const char *PreprocessorException::what() const noexcept

{
    std::stringstream error;
    error << "Preprocessor Error: ";
    error << m_errorString;

    return error.str().c_str();
}
