#include "valuestmt.hpp"

#include "intstmt.hpp"
#include "boolstmt.hpp"
#include "conststmt.hpp"
#include "executeforstmt.hpp"

using namespace gdsl;

const std::string ValueStmt::ASSIGN = "=";
const std::string ValueStmt::VALUESTMT_IN = "in";

const std::string ValueStmt::INT_TYPE = "int";
const std::string ValueStmt::BOOL_TYPE = "bool";

const char ValueStmt::RIGHT_BRACK = ']';
const char ValueStmt::LEFT_BRACK = '[';
const char ValueStmt::PRE_COMMA = ',';

std::shared_ptr<gdsl::ValueStmt> gdsl::ValueStmt::parseLine(std::string &ppLine)
{
    std::string name = PPUtils::pop(ppLine);

    if(!PPUtils::isName(name))
        throw PreprocessorException(name + " is not a name.");

    std::string op = PPUtils::pop(ppLine);

    if(op == ASSIGN)
        return parseAssign(ppLine, name);
    else if(op == VALUESTMT_IN)
        return parseIn(ppLine, name);
    else
        throw PreprocessorException("Unknown operator " + op);
}

std::shared_ptr<gdsl::ValueStmt> gdsl::ValueStmt::parseAssign(std::string &ppLine, const std::string& name)
{
    std::string value = PPUtils::pop(ppLine);

    if(!ppLine.empty())
        throw PreprocessorException("The line is empty.");

    if(value == INT_TYPE)
        return std::make_shared<IntStmt>(name);
    else if(value == BOOL_TYPE)
        return std::make_shared<BoolStmt>(name);
    else if(PPUtils::isNumber(value))
        return std::make_shared<ConstStmt>(name, PPUtils::toInt(value));
    else
        throw PreprocessorException("Could not recognize type of " + value);
}

std::shared_ptr<gdsl::ValueStmt> gdsl::ValueStmt::parseIn(std::string &ppLine, const std::string& name)
{
    auto token = PPUtils::popFirst(ppLine);

    if(token != LEFT_BRACK)
        throw PreprocessorException("Expected opening bracket, got " + token);

    std::string start = PPUtils::pop(ppLine, PRE_COMMA, false);

    token = PPUtils::popFirst(ppLine);
    if(token != PRE_COMMA)
        throw PreprocessorException("Expected comma, got " + token);

    std::string end = PPUtils::pop(ppLine, RIGHT_BRACK, false);

    token = PPUtils::popFirst(ppLine);
    if(token != RIGHT_BRACK)
        throw PreprocessorException("Expected closing bracket, got " + token);

    if(!PPUtils::isName(name))
        throw PreprocessorException(name + " is not a name");

    if(!PPUtils::isNumber(start))
        throw PreprocessorException(start + " is not a number");

    if(!PPUtils::isNumber(end))
        throw PreprocessorException(end + " is not a number");

    return std::make_shared<ExecuteForStmt>(name, start, end);
}
