#ifndef VALUESTMT_HPP
#define VALUESTMT_HPP

#include <string>
#include <memory>
#include <iostream>
#include <algorithm>
#include <sstream>

#include "../parser/gdslexception.hpp"
#include "preprocessorutilities.hpp"

namespace gdsl
{
class IntStmt;
class BoolStmt;
class ExecuteForStmt;
class ConstStmt;

class ValueStmt
{
    static const std::string ASSIGN;
    static const std::string VALUESTMT_IN;

    static const std::string INT_TYPE;
    static const std::string BOOL_TYPE;

    static const std::string PRE_IN;
    static const char RIGHT_BRACK;
    static const char LEFT_BRACK;
    static const char PRE_COMMA;

public:
    enum class RET_TYPE {PPINT, PPBOOL, PPCONST};

    static std::shared_ptr<ValueStmt> parseLine(std::string& ppLine);

    ValueStmt(std::string name) :
        m_name(name)
    {}

    virtual void setValue(const std::string&) {}
    virtual bool next() {return false;}
    virtual int value() const = 0;
    virtual int start() const {return value();}
    virtual int end() const {return value();}

    virtual std::string name() {return m_name;}
    virtual std::string valueStartStr() const {return std::to_string(start());}
    virtual std::string valueEndStr() const {return std::to_string(end());}
    virtual RET_TYPE returnType() const = 0;

protected:
    std::string m_name;

private:
    static std::shared_ptr<ValueStmt> parseAssign(std::string& ppLine, const std::string& name);
    static std::shared_ptr<ValueStmt> parseIn(std::string& ppLine, const std::string& name);
};
}
#endif // VALUESTMT_HPP
