#include "buttoncolorchooser.hpp"

ButtonColorChooser::ButtonColorChooser(std::shared_ptr<TreeItem> item, bool useOrigColor, QWidget *parent) :
    QPushButton(parent),
    m_useOrigColor(useOrigColor),
    m_origColor(item->origColor()),
    m_autoColor(item->color()),
    m_item(item)
{
    if(m_useOrigColor)
        m_currentColor = QColor(m_origColor);
    else
        m_currentColor = QColor(m_autoColor);

    auto stylesheet = QString("background-color: %1; border : 1px solid;").arg(m_currentColor.name());
    setStyleSheet(stylesheet);
    update();

    connect(this, SIGNAL(clicked()), this, SLOT(on_pushButton_clicked()));
}

ButtonColorChooser::~ButtonColorChooser()
{
}

void ButtonColorChooser::colorModeChanged(bool useOrigColor)
{
    if(m_useOrigColor != useOrigColor)
    {
        m_useOrigColor = useOrigColor;
        updateColor();
    }
}

void ButtonColorChooser::on_pushButton_clicked()
{
    auto color = QColorDialog::getColor(m_currentColor);

    if(color.isValid())
        m_currentColor = color;
    else
        return;

    m_item->setColor(m_currentColor.name(QColor::HexRgb));
    updateColor();

    emit colorChanged(m_item);
}

void ButtonColorChooser::updateColor()
{
    if(m_useOrigColor)
        m_currentColor = m_origColor;
    else
        m_currentColor = m_autoColor;

    auto stylesheet = QString("background-color: %1; border : 1px solid;").arg(m_currentColor.name());
    setStyleSheet(stylesheet);
    update();
}
