#include "gdslconstantswidget.hpp"
#include "ui_gdslconstantswidget.h"

#include "createvariabledialog.hpp"

GDSLConstantsWidget::GDSLConstantsWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::GDSLConstantsWidget)
{
    ui->setupUi(this);
}

GDSLConstantsWidget::~GDSLConstantsWidget()
{
    delete ui;
}

QVector<QString> GDSLConstantsWidget::preProcessorStrings() const
{
    QVector<QString> ppStrings;

    for(int i = 0; i < ui->constants->count(); ++i)
    {
        auto constant = ui->constants->item(i);
        ppStrings.append(constant->data(Qt::UserRole).toString());
    }

    return ppStrings;
}

void GDSLConstantsWidget::reset()
{
    ui->constants->clear();
}

void GDSLConstantsWidget::addConstant(const QString &name, const QString &value)
{
    if(name == "" || value == "")
        return;

    QListWidgetItem* item = new QListWidgetItem(ui->constants);
    item->setText(tr("%1 = %2").arg(name).arg(value));
    item->setData(Qt::UserRole, QString("%set %1 = %2").arg(name).arg(value));
}

void GDSLConstantsWidget::on_addButton_clicked()
{
    CreateVariableDialog dialog(this);
    if(dialog.exec() == QDialog::Rejected)
        return;

    addConstant(dialog.name(), dialog.value());
}

void GDSLConstantsWidget::on_pushButton_clicked()
{
    auto selectedItems = ui->constants->selectedItems();

    for(auto item : selectedItems)
    {
        int row = ui->constants->row(item);
        ui->constants->takeItem(row);
    }

    qDeleteAll(selectedItems);
}
