#include "gdsldisplayresult.hpp"
#include "ui_gdsldisplayresult.h"

const QList<QColor> GDSLDisplayResult::PLEASING_COLORS = QList<QColor>() << QColor("#6FD0C9")
                 << QColor("#EE7737")
                 << QColor("#70E143")
                 << QColor("#BAB056")
                 << QColor("#61DF8D")
                 << QColor("#82AEDB")
                 << QColor("#D0DD41")
                 << QColor("#D7975F")
                 << QColor("#75B544")
                 << QColor("#DFA930")
                 << QColor("#79B987")
                 << QColor("#C4E68A");


GDSLDisplayResult::GDSLDisplayResult(std::vector<std::shared_ptr<GDSLVariableInfo> > varInfos, int rows, int cols, QWidget *parent) :
    QWidget(parent),
    ui(new Ui::GDSLDisplayResult)
{
    ui->setupUi(this);

    m_scene = new GraphScene(this);
    ui->view->setScene(m_scene);

    setVariableInfo(varInfos, rows, cols);
}

GDSLDisplayResult::~GDSLDisplayResult()
{
    delete m_scene;
    delete ui;
}

void GDSLDisplayResult::setVariableInfo(std::vector<std::shared_ptr<GDSLVariableInfo> > varInfos, int rows, int cols)
{
    m_varInfos = varInfos;
    m_rows = rows;
    m_cols = cols;

    ui->table->clearTable();
    m_scene->clear();
    auto gridItem = m_scene->drawGrid(rows, cols);

    int i = 0; int j = 0;
    for(auto var : m_varInfos)
    {
        if(var->drawable())
        {
            var->setColorForNodes(PLEASING_COLORS[i].name(QColor::HexRgb));
            var->setColorForEdges(PLEASING_COLORS[i].name(QColor::HexRgb));
        }

        ui->table->addItem(var);

        if(var->drawable())
        {
            auto layer = new GDSLVarLayer(var, gridItem, PLEASING_COLORS[i]);
            connect(ui->table, SIGNAL(checkStatesChanged(std::shared_ptr<TreeItem>)), layer, SLOT(checkStateChanged(std::shared_ptr<TreeItem>)));
            connect(ui->table, SIGNAL(colorChanged(std::shared_ptr<TreeItem>)), layer, SLOT(colorChanged(std::shared_ptr<TreeItem>)));
            connect(this, SIGNAL(settingsChanged()), layer, SLOT(settingsChanged()));

            m_scene->addLayer(layer);
        }

        i = (i + 1) % PLEASING_COLORS.size();
        j++;
    }

    connect(this, SIGNAL(settingsChanged()), ui->table, SLOT(settingsChanged()));
}

void GDSLDisplayResult::settingsChangedSlot()
{
    emit settingsChanged();
}
