#ifndef GDSLEDITOR_H
#define GDSLEDITOR_H

#include <QPlainTextEdit>
#include <QPainter>
#include <QRect>
#include <QTextBlock>
#include <QStringListModel>
#include <QListView>
#include <QCompleter>

#include <iostream>

class GDSLEditor : public QPlainTextEdit
{
    Q_OBJECT
public:
    explicit GDSLEditor(QWidget *parent = 0);
    ~GDSLEditor();

    void lineNumberAreaPaintEvent(QPaintEvent *event);
    int lineNumberAreaWidth();

protected:
    void resizeEvent(QResizeEvent *event);
    void keyPressEvent(QKeyEvent *e);

private slots:
    void updateLineNumberAreaWidth(int ) {setViewportMargins(lineNumberAreaWidth(), 0, 0, 0);}
    void highlightCurrentLine();
    void updateLineNumberArea(const QRect &rect, int dy);
    void complete();
    void complete(QString str);
    void updateVariableCompletion();

private:
    bool currentLineIsFor() const;

    QWidget *m_lineNumberArea;

    QCompleter* m_completer;
    QStringListModel* m_model;
    QStringList m_standardCompletions;

public slots:

};

class LineNumberArea : public QWidget
{
public:
    LineNumberArea(GDSLEditor *editor) : QWidget(editor) {
        m_editor = editor;
    }

    QSize sizeHint() const {
        return QSize(m_editor->lineNumberAreaWidth(), 0);
    }

protected:
    void paintEvent(QPaintEvent *event) {
        m_editor->lineNumberAreaPaintEvent(event);
    }

private:
    GDSLEditor *m_editor;
};

#endif // GDSLEDITOR_H
