#include "gdslhighlighter.h"

const QColor GDSLHighlighter::SET_COLOR     = QColor(tr("#4fa618"));
const QColor GDSLHighlighter::STMT_COLOR    = QColor(tr("#19198c"));
const QColor GDSLHighlighter::COMMENT_COLOR = QColor(tr("#b59c00"));
const QColor GDSLHighlighter::MAKRO_COLOR   = QColor(Qt::gray);

GDSLHighlighter::GDSLHighlighter(QTextDocument *parent) :
    QSyntaxHighlighter(parent)
{
    HighlightingRule rule;

    m_defaultVariableFormat.setForeground(SET_COLOR);
    QStringList m_defaultVariables;
    m_defaultVariables << "#?\\bcols\\b"
                       << "#?\\brows\\b"
                       << "\\bnodes\\b"
                       << "\\bedges\\b"
                       << "\\bobjects\\b";

    foreach(const QString& pattern, m_defaultVariables)
    {
        rule.pattern = QRegExp(pattern);
        rule.format = m_defaultVariableFormat;
        m_highlightingRules.append(rule);
    }

    m_keywordFormat.setForeground(STMT_COLOR);
    //m_keywordFormat.setFontWeight(QFont::Bold);
    QStringList m_keywordPatterns;
    m_keywordPatterns << "\\bdef\\b"
                      << "\\bfor\\b"
                      << "\\bin\\b"
                      << "\\bsum\\b"
                      << "\\bend\\b"
                      << "\\bwith\\b"
                      << "\\bcolor\\b"
                      << "\\blabel\\b"
                      << "\\bdo\\b";

    foreach(const QString& pattern, m_keywordPatterns)
    {
        rule.pattern = QRegExp(pattern);
        rule.format = m_keywordFormat;
        m_highlightingRules.append(rule);
    }

    m_singleLineCommentFormat.setForeground(COMMENT_COLOR);
    rule.pattern = QRegExp("//[^\n]*");
    rule.format = m_singleLineCommentFormat;
    m_highlightingRules.append(rule);

    m_makroFormat.setForeground(MAKRO_COLOR);
    rule.pattern = QRegExp("%[^\n]*");
    rule.format = m_makroFormat;
    m_highlightingRules.append(rule);
}

void GDSLHighlighter::highlightBlock(const QString &text)
{
    QRegExp forbbidenPatterns("\\b(without (nodes|edges)|incident edges)\\b");

    foreach (const HighlightingRule &rule, m_highlightingRules)
    {
        QRegExp expression(rule.pattern);
        int index = expression.indexIn(text);
        int forbbidenIndex = forbbidenPatterns.indexIn(text);
        int length;

        while (index >= 0)
        {
            if(forbbidenIndex >= 0 && forbbidenIndex <= index)
            {
                length = forbbidenPatterns.matchedLength();

                index = expression.indexIn(text, forbbidenIndex + length);
                forbbidenIndex = forbbidenPatterns.indexIn(text, index);
            }
            else
            {
                length = expression.matchedLength();
                setFormat(index, length, rule.format);

                forbbidenIndex = forbbidenPatterns.indexIn(text, index + length);
                index = expression.indexIn(text, index + length);
            }
        }
    }
}
