#ifndef GDSLMAKROCHOOSER_HPP
#define GDSLMAKROCHOOSER_HPP

#include <QWidget>
#include <QFile>
#include <QFileInfo>
#include <QDir>
#include <QString>
#include <QStringList>
#include <QTreeWidgetItem>
#include <QVector>
#include <QLabel>
#include <QVBoxLayout>
#include <QSpinBox>
#include <QLineEdit>
#include <QSpacerItem>
#include <QGroupBox>
#include <QTextEdit>
#include <QSettings>

#include "../GDSL/preprocessor/gdslpreprocessor.hpp"
#include "nameedit.hpp"
#include "intedit.hpp"
#include "booledit.hpp"

#include <QDebug>

#include <string>

namespace Ui {
class GDSLMakroChooser;
}

class Makro : public QWidget
{
    Q_OBJECT

public:
    explicit Makro(QString makro, QString makroDir, QString makroFile, QTreeWidgetItem* correspondingItem, QWidget *parent = 0);

    QString useString();
    QString displayName() {return m_name;}
    QString fileName() {return m_fileName;}
    void increaseUseCounter() {++m_counter;}
    void decreaseUseCounter() {--m_counter;}
    int useCounter() const {return m_counter;}
    bool checkedByUser() const {return m_userChecked;}
    void setUserChecked(bool userChecked) {m_userChecked = userChecked;}

    QVector<QString> dependencies() {return m_dependencies;}
    QTreeWidgetItem* treeItem() {return m_item;}

private:
    QString m_makro;
    QString m_makroDir;
    QString m_name;
    QString m_fileName;
    QVector<QString> m_dependencies;
    QTreeWidgetItem* m_item;
    int m_counter = 0;
    bool m_userChecked = false;

    QVector<IntEdit*> m_intEdits;
    QVector<BoolEdit*> m_boolEdits;
    QVector<NameEdit*> m_nameEdits;
};

class GDSLMakroChooser : public QWidget
{
    Q_OBJECT

public:
    explicit GDSLMakroChooser(QWidget *parent = 0);
    ~GDSLMakroChooser();

    void loadMakroList(const QString path = "", QTreeWidgetItem *parent = nullptr);
    void loadFromDependencyGraph(const gdsl::DependencyGraph& depGraph);
    std::vector<std::string> processedMakros() const;

public slots:
    void makroClicked(QTreeWidgetItem* item, int);
    void makroChanged(QTreeWidgetItem* item, int column);

private:
    void findAndCheckItem(const QString& name);

    Ui::GDSLMakroChooser *ui;

    QLabel* m_emptyLabel;
    QVector<Makro*> m_makros;
    QVector<QTreeWidgetItem*> m_checkedItems;
    Makro* m_currentMakro;
    QString m_makroPath;
};

#endif // GDSLMAKROCHOOSER_HPP
