#include "gdslrectitem.hpp"

//GDSLRectItem::GDSLRectItem(std::shared_ptr<TreeItem> item, QGraphicsItem *parent) :
//    QGraphicsItem(parent),
//    m_item(item)
//{
//    setBrushToItemColor();
//}

GDSLRectItem::GDSLRectItem(std::shared_ptr<TreeItem> item, const QRectF &rect, QGraphicsItem *parent) :
    QGraphicsItem(parent),
    m_item(item),
    m_boundingRect(rect)
{
    setBrushToItemColor();
}

GDSLRectItem::GDSLRectItem(std::shared_ptr<TreeItem> item, qreal x, qreal y, qreal width, qreal height, QGraphicsItem *parent) :
    QGraphicsItem(parent),
    m_item(item)
{
    setBrushToItemColor();
    m_boundingRect = QRectF(x, y, width, height);
}

void GDSLRectItem::checkIfVisible()
{
    if(m_item->visible())
    {
        show();
        int row = m_item->coords().second;
        int column = m_item->coords().first;

        for(auto it = m_item->parent()->childrenBegin(); it < m_item->parent()->childrenEnd(); ++it)
        {
            std::shared_ptr<TreeItem> treeItem = *it;

            int diffRow = treeItem->coords().second - row;
            int diffCol = treeItem->coords().first - column;

            if(diffRow == -1 && diffCol == -1)
            {
                if(treeItem->visible())
                    m_graphicItems[0]->show();
                else
                    m_graphicItems[0]->hide();
            }
            else if(diffRow == -1 && diffCol == 0)
            {
                if(treeItem->visible())
                    m_graphicItems[1]->show();
                else
                    m_graphicItems[1]->hide();
            }
            else if(diffRow == -1 && diffCol == 1)
            {
                if(treeItem->visible())
                    m_graphicItems[2]->show();
                else
                    m_graphicItems[2]->hide();
            }
            else if(diffRow == 0 && diffCol == -1)
            {
                if(treeItem->visible())
                    m_graphicItems[3]->show();
                else
                    m_graphicItems[3]->hide();
            }
            else if(diffRow == 0 && diffCol == 1)
            {
                if(treeItem->visible())
                    m_graphicItems[5]->show();
                else
                    m_graphicItems[5]->hide();
            }
            else if(diffRow == 1 && diffCol == -1)
            {
                if(treeItem->visible())
                    m_graphicItems[6]->show();
                else
                    m_graphicItems[6]->hide();
            }
            else if(diffRow == 1 && diffCol == 0)
            {
                if(treeItem->visible())
                    m_graphicItems[7]->show();
                else
                    m_graphicItems[7]->hide();
            }
            else if(diffRow == 1 && diffCol == 1)
            {
                if(treeItem->visible())
                    m_graphicItems[8]->show();
                else
                    m_graphicItems[8]->hide();
            }
        }
    }
    else
        hide();
}

void GDSLRectItem::checkColor()
{
    if(m_item->parent()->type() == TreeItem::TreeItemType::NODE_ID || m_item->parent()->type() == TreeItem::TreeItemType::EDGE_ID)
    {
        if(m_useOrigColor)
            setBrushToOrigItemColor();
        else
            setBrushToItemColor();
    }
}

void GDSLRectItem::setBrush(QBrush brush)
{
    for(auto item : m_graphicItems)
    {
        item->setBrush(brush);
        item->setPen(QPen(brush.color()));
    }
}

void GDSLRectItem::createRectItems()
{
    QRectF singleItemRect(0, 0, m_boundingRect.width()/3, m_boundingRect.height()/3);

    for(int i = 0; i < 3; ++i)
        for(int j = 0; j < 3; ++j)
        {
            auto item = new QGraphicsRectItem(singleItemRect, this);
            item->moveBy(i*m_boundingRect.width()/3, j*m_boundingRect.height()/3);
            m_graphicItems.append(item);
            initShowHide(item, i, j);
        }
}

void GDSLRectItem::initShowHide(QGraphicsRectItem *item, int row, int column)
{
    if(row == 1 && column == 1)
        item->show();
    else
        item->hide();
}

void GDSLColumnItem::initShowHide(QGraphicsRectItem *item, int, int column)
{
    if(column == 0)
        item->show();
    else
        item->hide();
}


void GDSLRowItem::initShowHide(QGraphicsRectItem *item, int row, int)
{
    if(row == 0)
        item->show();
    else
        item->hide();
}


void GDSLLineItem::checkIfVisible()
{
    if(m_item->visible())
        show();
    else
        hide();
}
