#ifndef GDSLRECTITEM_HPP
#define GDSLRECTITEM_HPP

#include <QGraphicsRectItem>
#include <QVector>
#include <QString>
#include <QTreeWidgetItem>
#include <QGraphicsGridLayout>
#include <QPainter>

#include <memory>

#include "gdslvariableinfo.hpp"

#include <iostream>

class GDSLLineItem;
class GDSLRowItem;
class GDSLColumnItem;

class GDSLRectItem : public QGraphicsItem
{
    friend class GDSLLineItem;

public:
//    GDSLRectItem(std::shared_ptr<TreeItem> item, QGraphicsItem* parent = 0);
    GDSLRectItem(std::shared_ptr<TreeItem> item, const QRectF & rect, QGraphicsItem * parent = 0);
    GDSLRectItem(std::shared_ptr<TreeItem> item, qreal x, qreal y, qreal width, qreal height, QGraphicsItem * parent = 0);

    void paint(QPainter*, const QStyleOptionGraphicsItem*, QWidget*) {}
    QRectF boundingRect() const {return m_boundingRect;}

    std::shared_ptr<TreeItem> item() {return m_item;}
    void setItem(std::shared_ptr<TreeItem> item) {m_item = item;}
    virtual void checkIfVisible();
    void checkColor();
    void setUseOrigColor(bool useOrigColor) {m_useOrigColor = useOrigColor; checkColor();}

    void setBrush(QBrush brush);
    void createRectItems();

private:
    void setBrushToItemColor() {setBrush(QBrush(QColor(m_item->parent()->color())));}
    void setBrushToOrigItemColor() {setBrush(QBrush(QColor(m_item->parent()->origColor())));}
    virtual void initShowHide(QGraphicsRectItem* item, int row, int column);

    std::shared_ptr<TreeItem> m_item;
    bool m_useOrigColor = true;

    QBrush m_brush;
    QRectF m_boundingRect;
    QVector<QGraphicsRectItem*> m_graphicItems;
};

class GDSLLineItem : public GDSLRectItem
{
    friend class GDSLColumnItem;
    friend class GDSLRowItem;

public:
    GDSLLineItem(std::shared_ptr<TreeItem> item, const QRectF & rect, QGraphicsItem * parent = 0) : GDSLRectItem(item, rect, parent) {}
    GDSLLineItem(std::shared_ptr<TreeItem> item, qreal x, qreal y, qreal width, qreal height, QGraphicsItem * parent = 0) : GDSLRectItem(item, x, y, width, height, parent) {}

    virtual void checkIfVisible();
};

class GDSLColumnItem : public GDSLLineItem
{

public:
    GDSLColumnItem(std::shared_ptr<TreeItem> item, const QRectF & rect, QGraphicsItem * parent = 0) : GDSLLineItem(item, rect, parent) {}
    GDSLColumnItem(std::shared_ptr<TreeItem> item, qreal x, qreal y, qreal width, qreal height, QGraphicsItem * parent = 0) : GDSLLineItem(item, x, y, width, height, parent) {}

private:
    virtual void initShowHide(QGraphicsRectItem* item, int, int column);
};

class GDSLRowItem : public GDSLLineItem
{

public:
    GDSLRowItem(std::shared_ptr<TreeItem> item, const QRectF & rect, QGraphicsItem * parent = 0) : GDSLLineItem(item, rect, parent) {}
    GDSLRowItem(std::shared_ptr<TreeItem> item, qreal x, qreal y, qreal width, qreal height, QGraphicsItem * parent = 0) : GDSLLineItem(item, x, y, width, height, parent) {}

private:
    virtual void initShowHide(QGraphicsRectItem* item, int row, int);
};

#endif // GDSLRECTITEM_HPP
