#ifndef GDSLUIPROGRAM_HPP
#define GDSLUIPROGRAM_HPP

#include <string>
#include <memory>
#include <unordered_map>
#include "ogdf/basic/GraphAttributes.h"
#include "../GDSL/parser/randomaccessgraph.hpp"
#include "gurobi_c++.h"
#include "gdslvariableinfo.hpp"

namespace gdsl
{
class GDSLProgram;
class GDSLVariable;
class GDSLPreprocessor;
class Grid;
}

class GDSLuiProgram
{
public:
    typedef std::unordered_map<int, std::vector<std::shared_ptr<GDSLVariableInfo> > > ResultMap;

    GDSLuiProgram() {}
    ~GDSLuiProgram() {}
    void preprocess(std::string& name, std::string& program, std::string& makroDir);
    void parse(std::string& name, std::string& program);
    void preprocessAndParse(std::string& name, std::string& program, std::string& makroDir);
    std::vector< std::shared_ptr<GRBModel> > run(std::shared_ptr<gdsl::RandomAccessGraph> raGraph, std::shared_ptr<gdsl::Grid> grid, std::shared_ptr<GRBEnv> env);

    ResultMap variables() const;
    int numRuns() const {return m_numRuns;}
    std::shared_ptr<gdsl::GDSLPreprocessor> preprocessor() const;

    std::string parseErrorTitle() const {return m_parseErrorTitle;}
    std::string parseErrorMessage() const {return m_parseErrorMessage;}
    bool parseFinishedWithError() const {return m_parseFinishedWithError;}

    std::string runErrorTitle(int run) const
    {
        if(run >= numRuns())
            return "";

        return m_errorTitle.at(run);
    }
    std::string runErrorMessage(int run) const
    {
        if(run >= numRuns())
            return "";

        return m_errorMessage.at(run);
    }
    bool runFinishedWithError(int run) const
    {
        if(run >= numRuns())
            return false;

        return m_finishedWithError[run];
    }

private:    
    void setRunError(int run, std::string title, std::string message);
    void setParseError(std::string title, std::string message);
    std::shared_ptr<GRBModel> runOneTime(std::shared_ptr<gdsl::RandomAccessGraph> raGraph, std::shared_ptr<gdsl::Grid> grid, std::shared_ptr<GRBEnv> env);

    ResultMap m_varInfos;
    std::shared_ptr<gdsl::GDSLProgram> m_program;
    std::shared_ptr<gdsl::GDSLPreprocessor> m_preprocessor;
    int m_numRuns = 0;

    std::unordered_map<int, std::string> m_errorTitle;
    std::unordered_map<int, std::string> m_errorMessage;
    std::vector<bool> m_finishedWithError;

    std::string m_parseErrorTitle;
    std::string m_parseErrorMessage;
    bool m_parseFinishedWithError = false;
};

#endif // GDSLUIPROGRAM_HPP
