#ifndef GDSLVARIABLESTABLE_HPP
#define GDSLVARIABLESTABLE_HPP

#include <QWidget>
#include <QTreeWidgetItem>
#include <QLabel>
#include <QPushButton>
#include <QVariant>
#include <QDebug>

#include <vector>

#include "gdslvariableinfo.hpp"
#include "buttoncolorchooser.hpp"
#include "settingsdialog.hpp"

namespace Ui {
class GDSLVariablesTable;
}

class VariablesTableItem : public QTreeWidgetItem
{
public:
    VariablesTableItem(std::shared_ptr<TreeItem> gdslTreeItem) : QTreeWidgetItem(QTreeWidgetItem::UserType) {m_gdslTreeItem = gdslTreeItem;}
    VariablesTableItem(QTreeWidget* parent, std::shared_ptr<TreeItem> gdslTreeItem) : QTreeWidgetItem(parent, QTreeWidgetItem::UserType) {m_gdslTreeItem = gdslTreeItem;}
    VariablesTableItem(VariablesTableItem* parent, std::shared_ptr<TreeItem> gdslTreeItem) : QTreeWidgetItem(parent, QTreeWidgetItem::UserType) {m_gdslTreeItem = gdslTreeItem;}

    std::shared_ptr<TreeItem> gdslTreeItem() {return m_gdslTreeItem;}
    void setTreeItemVisibility(bool visible) {m_gdslTreeItem->setVisible(visible);}
    VariablesTableItem* parentTableItem()
    {
        return dynamic_cast<VariablesTableItem*>(parent());
    }

    VariablesTableItem* childTableItem(int i)
    {
        return dynamic_cast<VariablesTableItem*>(child(i));
    }

private:
    std::shared_ptr<TreeItem> m_gdslTreeItem;
};

class GDSLVariablesTable : public QWidget
{
    Q_OBJECT

    static const int VARIABLE_COLUMN    = 0;
    static const int DATA_COLUMN        = 1;
    static const int LABEL_COLUMN       = 2;
    static const int COLOR_COLUMN       = 3;

public:
    explicit GDSLVariablesTable(QWidget *parent = 0);
    ~GDSLVariablesTable();

    void addItem(std::shared_ptr<GDSLVariableInfo> info);
    void clearTable();
    QTreeWidgetItem* headItem(int i);

private:
    void readSettings();

    void showOrHideCoordinates();
    void showOrHideCoordinates(QTreeWidgetItem* item);

    void setColorsEnabled();
    void setColorsEnabled(QTreeWidgetItem* item);

    Ui::GDSLVariablesTable *ui;
    bool m_showDefaults = false;
    bool m_showZeros = true;
    bool m_showCoordinates = true;
    bool m_useOrigColors = true;

    void addTreeItem(VariablesTableItem* root, std::shared_ptr<TreeItem> tree);
    void toggleZeros(QTreeWidgetItem* item);
    void updateCheckState(VariablesTableItem* item, Qt::CheckState checkState);
    void updateParentCheckState(VariablesTableItem* item);

signals:
    void itemChanged(QTreeWidgetItem *item, int column);
    void checkStatesChanged(std::shared_ptr<TreeItem> item);
    void colorChanged(std::shared_ptr<TreeItem> item);
    void colorModeChanged(bool useOrigColors);

public slots:
    void itemChangedSlot(QTreeWidgetItem * item, int column = 0);
    void colorChangedSlot(std::shared_ptr<TreeItem> item);
    void toggleDefaults(bool state);
    void toggleZeros(bool);
    void settingsChanged() {readSettings();}

private slots:
    void expandedItem(QModelIndex);
};

#endif // GDSLVARIABLESTABLE_HPP
