#ifndef GDSLVARLAYER_HPP
#define GDSLVARLAYER_HPP

#include <QGraphicsObject>
#include <gdslrectitem.hpp>
#include <QBrush>
#include <QDebug>
#include <QTreeWidgetItem>

#include "gdslvariableinfo.hpp"
#include "qgriditem.hpp"
#include "settingsdialog.hpp"

class GDSLVarLayer : public QGraphicsObject
{
    Q_OBJECT
public:
    explicit GDSLVarLayer(std::shared_ptr<GDSLVariableInfo> var, QGridItem *grid, QBrush brush = QBrush(Qt::black));

    QRectF boundingRect() const {return m_rect;}
    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);

private:
    void readSettings();

    void drawTree(std::shared_ptr<TreeItem> item, QPainter *painter, const QStyleOptionGraphicsItem *option, bool edgeSubTree, QWidget *widget);
    void insertNode(int row, int column, std::shared_ptr<TreeItem> item, QPainter *painter,
                    const QStyleOptionGraphicsItem *option, bool edgeSubTree, QWidget *widget);
    void insertRowItem(int row, int column, std::shared_ptr<TreeItem> item, QPainter *painter,
                       const QStyleOptionGraphicsItem *option, bool edgeSubTree, QWidget *widget);
    void insertColItem(int row, int column, std::shared_ptr<TreeItem> item, QPainter *painter,
                       const QStyleOptionGraphicsItem *option, bool edgeSubTree, QWidget *widget);

    std::shared_ptr<GDSLVariableInfo> m_variable;
    int m_rows;
    int m_columns;
    QRectF m_rect;
    QBrush m_brush;

    bool m_layerChanged = false;
    bool m_visible = false;
    bool m_useOrigColors = false;

    void toggleVisible(bool visible);

signals:

public slots:
    void checkStateChanged(std::shared_ptr<TreeItem> item);
    void colorChanged(std::shared_ptr<TreeItem> item);
    void settingsChanged() {readSettings();}
};

#endif // GDSLVARLAYER_HPP
