#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QFileDialog>
#include <QTextStream>
#include <QDir>
#include <QFileInfo>
#include <QMessageBox>
#include <QPdfWriter>
#include <QSettings>
#include <QStandardPaths>
#include <QThread>

#include "ogdf/basic/GraphAttributes.h"

#include <iostream>
#include <memory>

#include "runworker.hpp"
#include "gdslparseworker.hpp"
#include "graphscene.h"
#include "gdslhighlighter.h"
#include "gdsldisplayresult.hpp"
#include "settingsdialog.hpp"
#include "graphpreview.hpp"

namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

private slots:
    void on_actionSave_Program_triggered();

    void on_actionLoad_Program_triggered();

    void on_actionLoad_Graph_triggered();

    void on_runButton_clicked();

    void on_actionExport_to_PNG_triggered();

    void on_actionExport_to_PDF_triggered();

    void parseFinished();
    void runFinished();
    void displayWorkerError();

    void on_actionSettings_triggered();

signals:
    void settingsChanged();

private:
    void checkSettings();
    std::string getProgramString();
    void parse(const std::string& programStr);
    void readPpStmts(const QString& fullProgram);

    Ui::MainWindow *ui;
    QSettings m_settings;

    GDSLHighlighter* m_highlighter;
    QDir m_lastGraphDirectory;
    QDir m_lastProgramDirectory;
    QDir m_lastExportDirectory;
    int m_currentResults = 0;

    std::shared_ptr<ogdf::Graph> m_graph;
    std::shared_ptr<ogdf::GraphAttributes> m_attr;

    std::shared_ptr<GDSLParseWorker> m_parser;
    std::shared_ptr<GDSLRunWorker> m_runner;

    void exportTo(QPaintDevice* device);
};

#endif // MAINWINDOW_H
