#include "qgriditem.hpp"

QGridItem::QGridItem(QGraphicsItem *parent) :
    QGraphicsObject(parent),
    m_pen(QPen())
{
    m_rows = 0;
    m_columns = 0;

    m_scale = 25.0;
    m_offset = 0;

    m_rect = QRectF(0, 0, m_scale, m_scale);
}

QGridItem::QGridItem(int rows, int columns, float scale, QPen pen, QGraphicsItem *parent) :
    QGraphicsObject(parent),
    m_pen(pen)
{
    m_rows = rows;
    m_columns = columns;

    m_scale = scale;
    m_offset = 0;

    m_changed = true;

    m_rect = QRectF(0, 0, m_scale, m_scale);
}

QRectF QGridItem::boundingRect() const
{
    return QRectF(-m_offset, -m_offset, m_scale * m_columns + m_offset, m_scale * m_rows + m_offset);
}

void QGridItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    if(m_changed)
    {
        for(int i = 0; i < m_rows; ++i)
        {
            for(int j = 0; j < m_columns; ++j)
            {
                QGraphicsRectItem* item = new QGraphicsRectItem(m_rect, this);
                item->moveBy(j * m_scale,i * m_scale);
                item->setPen(m_pen);
                item->paint(painter, option, widget);
            }
        }

        QGraphicsRectItem* item = new QGraphicsRectItem(boundingRect(), this);
        item->paint(painter, option, widget);
    }

    m_changed = false;
}

void QGridItem::clear()
{
    if(!childItems().empty())
    {
        for(auto item : childItems())
        {
            scene()->removeItem(item);
            delete item;
        }
    }
}
