#ifndef QGRIDITEM_HPP
#define QGRIDITEM_HPP

#include <QGraphicsObject>
#include <QPainter>
#include <QGraphicsScene>

#include <QDebug>

#include "qcrossitem.hpp"

class QGridItem : public QGraphicsObject
{
    Q_OBJECT
public:
    explicit QGridItem(QGraphicsItem *parent = 0);
    QGridItem(int rows = 0, int columns = 0, float scale = 25.0, QPen pen = QPen(), QGraphicsItem *parent = 0);

    QRectF boundingRect() const;
    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);

    int rows() const {return m_rows;}
    int columns() const {return m_columns;}
    void setChanged(bool state) {m_changed = state;}

    QRectF rectSize() const {return m_rect;}

private:
    void clear();

    int m_rows;
    int m_columns;

    bool m_changed = false;

    QPen m_pen;
    QRectF m_rect;

    float m_scale;
    float m_offset;

public slots:
    void rowsChanged(int rows) {clear(); m_rows = rows; m_changed = true; update(boundingRect()); scene()->setSceneRect(boundingRect());}
    void columnsChanged(int columns) {clear(); m_columns = columns; m_changed = true; update(boundingRect()); scene()->setSceneRect(boundingRect());}
    void rowsColumnsChanged(int rows, int columns) {clear(); m_rows = rows; m_columns = columns; m_changed = true; update(boundingRect()); scene()->setSceneRect(boundingRect());}
};

#endif // QGRIDITEM_HPP
