#ifndef RUNWORKER_HPP
#define RUNWORKER_HPP

#include <QObject>

#include <string>
#include <unordered_map>

#include "gdslvariableinfo.hpp"
#include "gdsluiprogram.hpp"
#include "../GDSL/parser/randomaccessgraph.hpp"

class GDSLRunWorker : public QObject
{
    Q_OBJECT
public:
    explicit GDSLRunWorker(std::shared_ptr<GDSLuiProgram> program, std::shared_ptr<gdsl::RandomAccessGraph> raGraph, std::shared_ptr<gdsl::Grid> grid, QObject *parent = 0);

    int numResults() const {return m_model.size();}
    int numRuns() const {return m_uiprogram->numRuns();}

    bool runFinishedWithError(int run) const
    {
        if(run >= numRuns())
            return false;

        return m_uiprogram->runFinishedWithError(run);
    }

    std::vector< std::shared_ptr<GDSLVariableInfo> >& variableInfo(int i) {return m_varInfos[i];}

    std::pair<std::string, std::string> getError(int run) const;

public slots:
    void process();

private:
    GDSLuiProgram::ResultMap m_varInfos;
    std::shared_ptr<gdsl::RandomAccessGraph> m_raGraph;
    std::shared_ptr<GDSLuiProgram> m_uiprogram;
    std::shared_ptr<gdsl::Grid> m_grid;

    std::shared_ptr<GRBEnv> m_env;
    std::vector< std::shared_ptr<GRBModel> > m_model;

signals:
    void finished();

public slots:

};

#endif // RUNWORKER_HPP
